/*
** Copyright (c) 2001  Hughes Technologies Pty Ltd.  All rights
** reserved.  
**
** Terms under which this software may be used or copied are
** provided in the  specific license associated with this product.
**
** Hughes Technologies disclaims all warranties with regard to this 
** software, including all implied warranties of merchantability and 
** fitness, in no event shall Hughes Technologies be liable for any 
** special, indirect or consequential damages or any damages whatsoever 
** resulting from loss of use, data or profits, whether in an action of 
** contract, negligence or other tortious action, arising out of or in 
** connection with the use or performance of this software.
**
**
** $Id: code.c,v 1.8 2002/01/08 00:17:25 bambi Exp $
**
*/

/*
** Module	: 
** Purpose	: 
** Exports	: 
** Depends Upon	: 
*/



/**************************************************************************
** STANDARD INCLUDES
**************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>

#include "config/config.h"
#include "ember.h"

#ifdef HAVE_UNISTD_H
#  include <unistd.h>
#endif
#ifdef HAVE_STRING_H
#  include <string.h>
#endif
#ifdef HAVE_STRINGS_H
#  include <strings.h>
#endif


/**************************************************************************
** MODULE SPECIFIC INCLUDES
**************************************************************************/

#include "parser.h"

/**************************************************************************
** GLOBAL VARIABLES
**************************************************************************/


/**************************************************************************
** PRIVATE ROUTINES
**************************************************************************/

static void _storePcode(script, code)
	ember	*script;
	e_code	*code;
{
	if (script->info.curFunction->lastPcode)
	{
		code->prev = script->info.curFunction->lastPcode;
		script->info.curFunction->lastPcode->next = code;
	}
	else
	{
		code->prev = NULL;
		script->info.curFunction->codePtr = code;
	}
	code->line = script->info.curLineNum;
	code->next = NULL;
	script->info.curFunction->lastPcode = code;
}

static e_code *findJumpBackTarget(script, code, label)
	ember	*script;
	e_code	*code;
	int	label;
{
	REG	e_code *curCode;

	curCode = code;
	while(curCode)
	{
		if (curCode->op == E_LABEL && curCode->arg.id == label)
			return(curCode);
		curCode = curCode->prev;
	}
	return(NULL);
}



static int _setJmpDestinations(script, code, label)
	ember	*script;
	e_code	*code;
{
	e_code	*curCode = code->prev;

	while(curCode)
	{
		if (curCode->arg.type == EMBER_ARG_LABEL)
		{
			if (curCode->arg.id == label)
			{
				curCode->arg.ptr = (void *)code;
			}
		}
		curCode = curCode->prev;
	}
	return(0);
}


static char *_opName(op)
	int	op;
{
	static	char buf[20];

	switch(op)
	{
		case E_STOR: return("STOR");
		case E_ARRAY_STOR: return("ARRAY_STOR");
		case E_ASSOC_STOR: return("ASSOC_STOR");
		case E_LOAD: return("LOAD");
		case E_CALL: return("CALL");
		case E_RET: return("RET");
		case E_ALU: return("ALU");
		case E_PUSH: return("PUSH");
		case E_POP: return("POP");
		case E_CMP: return("CMP");
		case E_JZ: return("JZ");
		case E_JMP: return("JMP");
		case E_JMPB: return("JMPB");
		case E_LABEL: return("LABEL");
		case E_CAST: return("CAST");
		case E_ARRAY_DEREF: return("ARRAY_DEREF");
		case E_ASSOC_DEREF: return("ASSOC_DEREF");
		case E_COUNT: return("COUNT");
		case E_BLOCK_IN: return("BLOCK_IN");
		case E_BLOCK_OUT: return("BLOCK_OUT");
		case E_RETURN: return("RETURN");
		case E_EXISTS: return("EXISTS");
		default: 
			snprintf(buf,20,"?? (%d)", op);
			return(buf);
	}
}

/**************************************************************************
** PUBLIC ROUTINES
**************************************************************************/



/*
** Label related routines
*/

int eCodePushLabel(script, label)
	ember	*script;
	int	label;
{
	e_label	*new;

	new = (e_label *)eMemMalloc(script,sizeof(e_label));
	if (!new)
		return(-1);
	new->label = label;
	new->next = script->labelStack;
	script->labelStack = new;
	return(0);
}



int eCodePopLabel(script)
	ember	*script;
{
	e_label	*next;
	int	label;

	if (script->labelStack == NULL)
	{
		strcpy(script->errorMsg,"Label stack underflow");
		return(-1);
	}
	next = script->labelStack->next;
	label = script->labelStack->label;
	eMemFree(script, script->labelStack);
	script->labelStack = next;
	return(label);
}


int eCodePushBreakLabel(script, label)
	ember	*script;
	int	label;
{
	e_label	*new;

	new = (e_label *)eMemMalloc(script,sizeof(e_label));
	if (!new)
		return(-1);
	new->label = label;
	new->next = script->breakStack;
	script->breakStack = new;
	return(0);
}



int eCodePopBreakLabel(script)
	ember	*script;
{
	e_label	*next;
	int	label;

	if (script->breakStack == NULL)
	{
		strcpy(script->errorMsg,"Break stack underflow");
		return(-1);
	}
	next = script->breakStack->next;
	label = script->breakStack->label;
	eMemFree(script, script->breakStack);
	script->breakStack = next;
	return(label);
}


int eCodePushContLabel(script, label)
	ember	*script;
	int	label;
{
	e_label	*new;

	new = (e_label *)eMemMalloc(script,sizeof(e_label));
	if (!new)
		return(-1);
	new->label = label;
	new->next = script->contStack;
	script->contStack = new;
	return(0);
}



int eCodePopContLabel(script)
	ember	*script;
{
	e_label	*next;
	int	label;

	if (script->contStack == NULL)
	{
		strcpy(script->errorMsg,"Continue stack underflow");
		return(-1);
	}
	next = script->contStack->next;
	label = script->contStack->label;
	eMemFree(script, script->contStack);
	script->contStack = next;
	return(label);
}


int eCodePeekLabel(script)
	ember	*script;
{
	if (script->labelStack == NULL)
	{
		strcpy(script->errorMsg,"Label stack underflow on peek");
		return(-1);
	}
	return(script->labelStack->label);
}

int eCodeGetLabel(script)
	ember	*script;
{
	script->info.lastLabelID++;
	return(script->info.lastLabelID);
}



/*
** Instruction set related routines
*/

int eCodeStore(script, symId)
	ember	*script;
	int	symId;
{
	e_code	*new;

	new = (e_code *)malloc(sizeof(e_code));
	new->op = E_STOR;
	/* new->arg.id = symId; */
	new->arg.id = 0;
	new->arg.type = EMBER_ARG_VARIABLE;
	new->arg.ptr = (void *)eSymGetVariableById(script, symId);
	_storePcode(script, new);
	return(0);
}


int eCodeArrayStore(script, symId)
	ember	*script;
	int	symId;
{
	e_code	*new;

	new = (e_code *)malloc(sizeof(e_code));
	new->op = E_ARRAY_STOR;
	new->arg.id = symId;
	new->arg.type = EMBER_ARG_VARIABLE;
	new->arg.ptr = (void *)eSymGetVariableById(script, symId);
	_storePcode(script, new);
	return(0);
}


int eCodeAssocStore(script, symId)
	ember	*script;
	int	symId;
{
	e_code	*new;

	new = (e_code *)malloc(sizeof(e_code));
	new->op = E_ASSOC_STOR;
	new->arg.id = symId;
	new->arg.type = EMBER_ARG_VARIABLE;
	new->arg.ptr = (void *)eSymGetVariableById(script, symId);
	_storePcode(script, new);
	return(0);
}


int eCodePushLiteral(script, type, value)
	ember	*script;
	int	type;
	char	*value;
{
	int	id;
	e_code	*new;

	if(eSymCreateLiteral(script,type,value,&id) < 0)
		return(-1);
	new = (e_code *)malloc(sizeof(e_code));
	new->op = E_PUSH;
	new->arg.id = id;
	new->arg.type = EMBER_ARG_LITERAL;
	new->arg.ptr = (void *)eSymGetLiteral(script, id);
	_storePcode(script, new);
	return(0);
}

int eCodePushVariable(script, var)
	ember	*script;
	char	*var;
{
	e_stab	*sym;
	e_code	*new;

	sym = eSymGetVariableByName(script, var);
	if (sym == NULL)
		sym = eSymCreateVariable(script, var);
	if (sym == NULL)
		return(-1);
	if (sym == NULL)
		return(-1);
	new = (e_code *)eMemMalloc(script,sizeof(e_code));

	new->op = E_PUSH;
	/*new->arg.id = sym->id;*/
	new->arg.id = 0;
	new->arg.type = EMBER_ARG_VARIABLE;
	new->arg.ptr = (void *)sym;
	_storePcode(script, new);
	return(0);
}

int eCodePushMacro(script, macro)
	ember	*script;
	char	*macro;
{
	e_stab	*sym;
	e_code	*new;
	e_map	*map;

	map = eSymFindVariableMap(script, script->macros, macro);
	if (map == NULL)
	{
		sprintf(script->errorMsg,"Reference to undefined macro '%s'",
			macro);
		return(-1);
	}
	sym = eSymGetLiteral(script, map->id);
	if (sym == NULL)
		return(-1);

	new = (e_code *)eMemMalloc(script,sizeof(e_code));

	new->op = E_PUSH;
	/*new->arg.id = sym->id;*/
	new->arg.id = 0;
	new->arg.type = EMBER_ARG_MACRO;
	new->arg.ptr = (void *)sym;
	_storePcode(script, new);
	return(0);
}

int eCodeTypeCast(script, type)
	ember	*script;
	int	type;
{
	e_code	*new;
	e_type	*typePtr;

	typePtr = eTypeGetType(type);
	new = (e_code *)malloc(sizeof(e_code));
	new->op = E_CAST;
	new->arg.id = type;
	new->arg.type = EMBER_ARG_TYPE;
	new->arg.ptr = (void *)typePtr;
	_storePcode(script, new);
	return(0);
}


int eCodeAlu(script, op)
	ember	*script;
	int	op;
{
	e_code	*new;

	new = (e_code *)malloc(sizeof(e_code));
	new->op = E_ALU;
	new->arg.id = op;
	new->arg.type = EMBER_ARG_ALU;
	new->arg.ptr = NULL;
	_storePcode(script, new);
	return(0);
}

int eCodeLabel(script, label)
	ember	*script;
	int	label;
{
	e_code	*new;

	new = (e_code *)malloc(sizeof(e_code));
	new->op = E_LABEL;
	new->arg.id = label;
	new->arg.type = EMBER_ARG_LABEL;
	new->arg.ptr = NULL;
	_storePcode(script, new);
	_setJmpDestinations(script,new,label);
	return(0);
}

int eCodeJZ(script, label)
	ember	*script;
	int	label;
{
	e_code	*new;

	new = (e_code *)malloc(sizeof(e_code));
	new->op = E_JZ;
	new->arg.id = label;
	new->arg.type = EMBER_ARG_LABEL;
	new->arg.ptr = NULL;
	_storePcode(script, new);
	return(0);
}

int eCodeJMP(script, label)
	ember	*script;
	int	label;
{
	e_code	*new;

	new = (e_code *)malloc(sizeof(e_code));
	new->op = E_JMP;
	new->arg.id = label;
	new->arg.type = EMBER_ARG_LABEL;
	new->arg.ptr = NULL;
	_storePcode(script, new);
	return(0);
}


int eCodeJMPB(script, label)
	ember	*script;
	int	label;
{
	e_code	*new;

	new = (e_code *)malloc(sizeof(e_code));
	new->op = E_JMPB;
	new->arg.id = label;
	new->arg.type = EMBER_ARG_LABEL;
	_storePcode(script, new);
	new->arg.ptr = (void*)findJumpBackTarget(script, new, label);
	return(0);
}

int eCodeCMP(script, op)
	ember	*script;
	int	op;
{
	e_code	*new;

	new = (e_code *)malloc(sizeof(e_code));
	new->op = E_CMP;
	new->arg.id = op;
	new->arg.type = EMBER_ARG_COND;
	new->arg.ptr = NULL;
	_storePcode(script, new);
	return(0);
}


int eCodeBlockIn(script)
	ember	*script;
{
	e_code	*new;

	new = (e_code *)malloc(sizeof(e_code));
	new->op = E_BLOCK_IN;
	new->arg.id = 0;
	new->arg.type = 0;
	new->arg.ptr = NULL;
	_storePcode(script, new);
	return(0);
}


int eCodeBlockOut(script)
	ember	*script;
{
	e_code	*new;

	new = (e_code *)malloc(sizeof(e_code));
	new->op = E_BLOCK_OUT;
	new->arg.id = 0;
	new->arg.type = 0;
	new->arg.ptr = NULL;
	_storePcode(script, new);
	return(0);
}


int eCodeArrayDeref(script)
	ember	*script;
{
	e_code	*new;

	new = (e_code *)malloc(sizeof(e_code));
	new->op = E_ARRAY_DEREF;
	new->arg.id = 0;
	new->arg.type = 0;
	new->arg.ptr = NULL;
	_storePcode(script, new);
	return(0);
}


int eCodeAssocDeref(script)
	ember	*script;
{
	e_code	*new;

	new = (e_code *)malloc(sizeof(e_code));
	new->op = E_ASSOC_DEREF;
	new->arg.id = 0;
	new->arg.type = 0;
	new->arg.ptr = NULL;
	_storePcode(script, new);
	return(0);
}

int eCodeCount(script)
	ember	*script;
{
	e_code	*new;

	new = (e_code *)malloc(sizeof(e_code));
	new->op = E_COUNT;
	new->arg.id = 0;
	new->arg.type = 0;
	new->arg.ptr = NULL;
	_storePcode(script, new);
	return(0);
}


int eCodeExists(script, var)
	ember	*script;
	char	*var;
{
	int	id;
	e_code	*new;

	if(eSymCreateLiteral(script,EMBER_TYPE_TEXT,var,&id) < 0)
		return(-1);
	new = (e_code *)malloc(sizeof(e_code));
	new->op = E_EXISTS;
	new->arg.id = id;
	new->arg.type = EMBER_ARG_LITERAL;
	new->arg.ptr = (void *)eSymGetLiteral(script, id);
	_storePcode(script, new);
	return(0);
}

int eCodeReturn(script)
	ember	*script;
{
	e_code	*new;

	new = (e_code *)malloc(sizeof(e_code));
	new->op = E_RETURN;
	new->arg.id = 0;
	new->arg.type = 0;
	new->arg.ptr = NULL;
	_storePcode(script, new);
	return(0);
}


int eCodeCall(script, funct, checkReturn)
	ember	*script;
	char	*funct;
	int	checkReturn;
{
	e_funct	*ptr;
	e_code	*new;

	ptr = eSymGetFunctionByName(script, funct);
	if (ptr == NULL)
		return(-1);

	new = (e_code *)eMemMalloc(script,sizeof(e_code));

	new->op = E_CALL;
	new->arg.id = ptr->id;
	new->arg.type = EMBER_ARG_FUNCTION;
	new->arg.ptr = (void *)ptr;
	_storePcode(script, new);

	if (script->info.ignoreReturn && checkReturn)
	{
		/*
		** The return value of the function call is not
		** used.  Pop it off the stack so the stack doesn't
		** grow.
		*/
		new = (e_code *)eMemMalloc(script,sizeof(e_code));
		new->op = E_POP;
		new->arg.id = 0;
		new->arg.type = 0;
		new->arg.ptr = NULL;
		_storePcode(script, new);
		
	}
	return(0);
}


void eCodeDumpCode(script, code)
	ember	*script;
	e_code	*code;
{
	e_stab	*sym;
	e_funct	*funct;
	e_type	*type;
	e_code	*codeArg;
	int	op;
	static	char *alu[] = { "SHIFT_L","SHIFT_R","ADD","SUB","MUL","DIV",
				"MOD", "AND", "OR", "???"};
	static	char *cond[]= { "GE", "LE", "NE", "EQ", "GT", "LT",
				"RE", "AND", "OR", "???"};

	printf("   Op          Line             Arg         \n");
	printf(" ---------------------------------------------------\n");

	while(code)
	{
		printf("  %-8.8s     %-5d", _opName((int)code->op),code->line);
		switch(code->arg.type)
		{
			case EMBER_ARG_VARIABLE:
				sym = (e_stab*)code->arg.ptr;
				printf("Var %d %s", code->arg.id, sym->name);
				break;

			case EMBER_ARG_LITERAL:
				sym = (e_stab*)code->arg.ptr;
				printf("Lit %d ", code->arg.id);
				break;

			case EMBER_ARG_FUNCTION:
				funct = (e_funct*)code->arg.ptr;
				printf("Funct %d %s ", code->arg.id,
					funct->name);
				break;

			case EMBER_ARG_TYPE:
				type = (e_type*)code->arg.ptr;
				printf("Type %s ", type->name);
				break;

			case EMBER_ARG_ALU:
				op = code->arg.id - OP_SHIFT_L;
				printf("Op %s ", alu[op]);
				break;

			case EMBER_ARG_LABEL:
				printf("Label %d ", code->arg.id);
				if (code->op != E_LABEL)
				{
					codeArg = (e_code *)code->arg.ptr;
					if (codeArg == NULL)
						printf("  BAD DEST PTR");
					else
						printf("  Ptr-> %d",
							codeArg->arg.id);
				}
				break;

			case EMBER_ARG_COND:
				op = code->arg.id - COND_GE;
				printf("Cond %s ", cond[op]);
				break;
		}
		printf("\n");
		code = code->next;
	}
}



void eCodeFreeCode(script, code)
	ember	*script;
	e_code	*code;
{
	e_code	*curCode,
		*prevCode;

	curCode = code;
	while(curCode)
	{
		prevCode = curCode;
		curCode = curCode->next;
		free(prevCode);
	}
}

