/*
**
** Copyright (c) 2001  Hughes Technologies Pty Ltd
**
** Permission to use, copy, and distribute for non-commercial purposes,
** is hereby granted without fee, providing that the above copyright
** notice appear in all copies and that both the copyright notice and this
** permission notice appear in supporting documentation.
**
** This software is provided "as is" without any expressed or implied warranty.
**
**
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <sys/file.h>
#include <sys/types.h>

#include "config/config.h"
#include "regexp/regexp.h"
#include "ember.h"
#include "parser.h"

#ifdef HAVE_DLFCN_H
#  include <dlfcn.h>
#endif

#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif


int eDynLoadModule(script, module)
	ember	*script;
	char	*module;
{
	char	path[255],
		tmp[255];
	void	*modPtr;
	int	(*init)();
	

	/* Strip the quotes */
	if (*module == '"')
	{
		strcpy(tmp, module + 1);
		tmp[strlen(tmp) - 1] = 0;
	}
	else
	{
		strcpy(tmp, module);
	}

	/* Try the default and specified location */
	sprintf(path,"%s/modules/%s.so", INST_DIR, tmp);
#ifdef 	RTLD_GLOBAL
	modPtr = dlopen(path,RTLD_LAZY|RTLD_GLOBAL);
#else
	modPtr = dlopen(path,RTLD_LAZY);
#endif
	if (modPtr == NULL)
	{
		sprintf(path,"%s.so", tmp);
		modPtr = dlopen(path, 1);
	}
	if (modPtr == NULL)
	{
		sprintf(script->errorMsg,"Mod Load failed : %s", dlerror());
		return(-1);
	}
	init = (int(*)())dlsym(modPtr,"_init_mod");
	if (!init)
	{
		init = (int(*)())dlsym(modPtr,"init_mod");
		if (!init)
		{
			sprintf(script->errorMsg,"Mod Load failed : %s", 
				dlerror());
			return(-1);
		}
		script->info.dynStripLeader = 1;
	}
	if ((*init)(script, modPtr) < 0)
		return(-1);
	return(0);
}


int eDynAddModule(script, modPtr, module)
        ember   *script;
	void	*modPtr;
        e_extern module[];
{
        e_extern *curFunct;
	char	*symName;
        int     count;

        count = 0;
        curFunct = &module[count];
        while(curFunct->name)
        {
		if (script->info.dynStripLeader)
			symName = curFunct->functName + 1;
		else
			symName = curFunct->functName;
		curFunct->functPtr = (int (*)())dlsym(modPtr, symName);
                count++;
                curFunct = &module[count];
        }
        return(eAddModule(script, module, NULL));
}



#ifdef NOTDEF

int modLoadModuleFunctions(script, modPtr, functs)
	ember	*script;
	void	*modPtr;
        efunct_t *functs;
{
        efunct_t *cur;
        int     hash;
        fbucket_t       *new;
	char		*cp;

        if (!externInit)
        {
                externInit=1;
                fHash = (fbucket_t **) malloc(sizeof(void *) * NUM_HASH);
		bzero(fHash,NUM_HASH * sizeof(void*));
        }
        cur = functs;
        while(cur->name)
        {
		cp = cur->functName;
		if (stripLeader)
			cp++;
		cur->funct = (void(*)())dlsym(modPtr,cp);
		if (!cur->funct)
		{
			sprintf(errBuf,"Can't load module symbol '%s'",
				cur->functName);
			parseError(errBuf);
			return(-1);
		}
                hash = calcHash(cur->name);
                new = (fbucket_t *)malloc(sizeof(fbucket_t));
                (void)bzero(new,sizeof(fbucket_t));
                new->funct = cur;
                if (fHash[hash])
                {
                        new->next = fHash[hash];
                }
                fHash[hash] = new;
                cur++;
        }
	return(0);
}


#if defined(_OS_OS2) || defined(_OS_WIN32)

/*
 * get the callbacks from the executable, and assigns pointers
 * 1998 Yuri Dario <mc6530@mclink.it>
*/
int APIENTRY modLoadModuleCallbacks(cb)
	callback	*cb;
{
	runError = cb->runError;
	parseError = cb->parseError;
	yylex = cb->yylex;
	yyerror = cb->yyerror;
	yylineno = cb->yylineno;
	checkContentType = cb->checkContentType;

	return(0);
}

/*
 * calls checkContentType: used for dynamic loaded modules that can't
 * access pointers inside LITE.DLL
 * 1998 Yuri Dario <mc6530@mclink.it>
*/
void APIENTRY modCheckContentType(void)
{
   checkContentType();
}

/*
 * calls runError: used for dynamic loaded modules that can't
 * access pointers inside LITE.DLL
 * 1998 Yuri Dario <mc6530@mclink.it>
*/
void APIENTRY modRunError(msg)
	char	*msg;
{
   runError(msg);
}

#endif  /* _OS_OS2 or _OS_WIN32 */

#endif
