/*
** ember Header File
*/


/***********************************************************************
** Standard header preamble.  Ensure singular inclusion, setup for
** function prototypes and c++ inclusion
*/

#ifndef EMBER_H

#define EMBER_H 1

#if defined(__STDC__) || defined(__cplusplus)
#  define __ANSI_PROTO(x)       x
#else
#  define __ANSI_PROTO(x)       ()
#endif

#ifdef __cplusplus
extern "C" {
#endif



/***********************************************************************
** Macro Definitions
*/


/* Limit definitions */

#define	EMBER_NAME_LEN		30
#define	EMBER_HASH_LEN		32
#define	EMBER_ERR_LEN		160
#define	EMBER_TEXT_LEN		20480


/* Enumerations */

#define	EMBER_FILE_INPUT	1
#define EMBER_BUFFER_INPUT	2

#define	EMBER_ARG_VARIABLE	1
#define	EMBER_ARG_LITERAL	2
#define	EMBER_ARG_MACRO		3
#define	EMBER_ARG_FUNCTION	4
#define	EMBER_ARG_TYPE		5
#define	EMBER_ARG_ALU		6
#define	EMBER_ARG_LABEL		7
#define	EMBER_ARG_COND		8

#define EMBER_SCRIPT_FUNCTION	1
#define EMBER_MODULE_FUNCTION	2

#define EMBER_TYPE_INT 		1
#define EMBER_TYPE_UINT		2
#define EMBER_TYPE_TEXT		3
#define EMBER_TYPE_REAL		4
#define EMBER_TYPE_MONEY	5
#define EMBER_TYPE_ARRAY	6
#define EMBER_TYPE_ASSOC	7


/* Type shortcuts */

#define	REG			register
#define	EMBER_TRUE		1
#define EMBER_FALSE		0

typedef char			*C_PTR;
#undef	YYSTYPE
#define	YYSTYPE			C_PTR


/* Binary file infor */

#define	EMBER_BINARY_MAGIC	"EmBeR-HuGhEs"
#define	EMBER_BINARY_PROGRAM	1
#define	EMBER_BINARY_LIBRARY	2
#define EMBER_BINARY_VERSION	2

#define EMBER_BIN_LITERAL	1
#define EMBER_BIN_GLOBAL	2
#define EMBER_BIN_VARIABLE	3
#define EMBER_BIN_FUNCT		4
#define EMBER_BIN_ARG		5
#define EMBER_BIN_CODE		6


/*  Simulator intruction set */

#define E_STOR    	1
#define E_ARRAY_STOR   	2
#define E_ASSOC_STOR   	3
#define E_LOAD    	4
#define E_CALL    	5
#define E_RET     	6
#define E_ALU     	7
#define E_PUSH    	8
#define E_POP     	9 
#define E_CMP     	10
#define E_JZ      	11
#define E_JMP     	12
#define E_JMPB    	13
#define E_LABEL   	14
#define E_CAST    	15
#define E_ARRAY_DEREF  	16   
#define E_ASSOC_DEREF  	17   
#define E_COUNT   	18
#define	E_BLOCK_IN	19
#define E_BLOCK_OUT	20
#define E_RETURN	21
#define	E_EXISTS	22


/* Convenience macros (designated with and 'em' prefix */

#define	emUnpackSymbol(sym) (*sym->value.typePtr->unpack) (&sym->value)
#define emIsLiteral(name) (*name == 0)

#define	emTextArg(arg) (arg->value.value.textVal)
#define	emIntArg(arg) (arg->value.value.intVal)
#define	emUintArg(arg) (arg->value.value.uintVal)
#define	emRealArg(arg) (arg->value.value.realVal)
#define	emArrayArg(arg) (&arg->value)
#define emArgType(arg) (arg->value.type)


#define eDynamicAddModule(s,m,e) 	(*s->methods.addDynModule)(s,m,e)
#define eDynamicStoreObject(s,n,d,p) 	(*s->methods.storeObject)(s,n,d,p)
#define eDynamicFetchObject(s,n,d,i) 	(*s->methods.fetchObject)(s,n,d,i)
#define eDynamicDeleteObject(s,n,d,i) 	(*s->methods.deleteObject)(s,n,d,i)

#define eDynamicSetIntValue(s,p,v)	(*s->methods.setIntValue)(s,p,v)
#define eDynamicSetUintValue(s,p,v)	(*s->methods.setUintValue)(s,p,v)
#define eDynamicSetRealValue(s,p,v)	(*s->methods.setRealValue)(s,p,v)
#define eDynamicSetTextValue(s,p,v)	(*s->methods.setTextValue)(s,p,v)
#define eDynamicSetArrayElement(s,p,v,i)(*s->methods.setArrayElement)(s,p,v,i)
#define eDynamicGetArrayElement(s,p,i)(*s->methods.getArrayElement)(s,p,i)
#define eDynamicGetNumArrayElements(s,p) (*s->methods.getNumArrayElements)(s,p)
#define eDynamicInitArray(s,v)		(*s->methods.initArray)(s,v)
#define eDynamicFreeValue(s,v)		(*s->methods.freeValue)(s,v)

#define eDynamicCreateIntMacro(s,n,v)	(*s->methods.createIntMacro)(s,n,v)
#define eDynamicCreateTextMacro(s,n,v)	(*s->methods.createTextMacro)(s,n,v)
#define eDynamicCreateRealMacro(s,n,v)	(*s->methods.createRealMacro)(s,n,v)

#define eDynamicSetError(s,t,m)		(*s->methods.setError)(s,t,m)

#define	eLoadDynamicModule(s,m)		eDynLoadModule(s,m)

#define	EMBER_BRK	""

#define	eCreateVariable			eSymCreateVariable
#define	eSetIntValue(e,s,v) 		eSymSetIntValue(e,&s->value,v)
#define	eSetUntValue(e,s,v) 		eSymSetUintValue(e,&s->value,v)
#define	eSetRealValue(e,s,v) 		eSymSetRealValue(e,&s->value,v)
#define	eSetTextValue(e,s,v) 		eSymSetTextValue(e,&s->value,v)
#define	eGetVariableValue 		eSymGetVariableValue

/***********************************************************************
** Type Definitions
*/

typedef struct e_obj_s {
	int	id;
	char	module[EMBER_NAME_LEN + 1],
		descr[EMBER_NAME_LEN + 1];
	void	*obj;
	struct	e_obj_s *next;
} e_obj;


typedef struct {
	char    name[EMBER_NAME_LEN + 1];
        int     length;
        int     (*pack)();
        char*   (*unpack)();
        int     (*compare)();
        int 	(*math)();
        int     (*check)();
} e_type;


	

typedef	struct {
	char	initialised;
	int	type,
		length;
	e_type	*typePtr;
	union {
		char	*textVal;
		int	intVal;
		u_int	uintVal;
		double	realVal;
		void 	*arrayVal,
			*assocVal;
	} value;
} e_value;



typedef struct e_array_s {
	e_value	value;
	struct	e_array_s *next;
} e_array;

typedef struct e_assoc_s {
	char	*index;
	e_value	value;
	struct	e_assoc_s *next;
} e_assoc;

typedef struct stab_s {
	char	name[EMBER_NAME_LEN + 1];
	int	id;
	e_value	value;
	struct 	stab_s *next;
} e_stab;


typedef struct {
	int	result;
	char	*errMsg;
	e_stab	*variables;
} e_exec;


typedef struct code_s{
	int	op,
		line,
		file;
	struct	{
		int	type,
			id;
		void	*ptr;
	} arg;
	struct	code_s *next,
		*prev;
} e_code;


typedef struct namemap_s {
	char	name[EMBER_NAME_LEN + 1];
	int	id;
	struct	namemap_s *next;
} e_map;


typedef struct {
	char	*name;
	int	(*functPtr)();		/* Used for static linked */
	char	*functName;		/* Used for dynamic loaded */
	int	numArgs;
	int	argTypes[20];
} e_extern;

typedef struct e_arg_s {
	char	name[EMBER_NAME_LEN + 1];
	int	id,
		type;
	e_stab	*ptr;
	struct	e_arg_s *next;
} e_arg;

typedef struct funct_s {
	char	name[EMBER_NAME_LEN + 1];
	e_map	*variables[EMBER_HASH_LEN];
	e_arg	*args;
	int	type,
		id,
		numArgs;
	e_code	*codePtr,
		*lastPcode;
	e_extern *externPtr;
	struct	funct_s *next;
} e_funct;


typedef struct fstack_s {
	e_funct	*funct;
	struct fstack_s *next;
} e_fstck;


typedef struct {
	int	parseHtml,
		headersSent,
		responseSent;
	char	*contentType,
		*customHeader,
		*customResponse;
} e_www;


typedef struct {
        int	(*addDynModule)();
	int	(*storeObject)();
	void 	*(*fetchObject)();
	void 	*(*deleteObject)();
	int	(*setIntValue)();
	int	(*setUintValue)();
	int	(*setRealValue)();
	int	(*setTextValue)();
	int	(*setArrayElement)();
	int	(*createIntMacro)();
	int	(*createTextMacro)();
	int	(*createRealMacro)();
	e_value	*(*getArrayElement)();
	int	(*getNumArrayElements)();
	void	(*initArray)();
	void	(*freeValue)();
	void	(*setError)();
} e_meth;


typedef struct {
	int	curLineNum,
		commentStart,
		inComment,
		lastSymID,
		lastLitID,
		lastFunctID,
		lastLabelID,
		lastFileID,
		curBlock,
		curFileID,
		lastObjectID,
		lexStartState,
		dynStripLeader;
	char	traceFlag,
		ignoreReturn;
	e_funct	*curFunction;
} e_info;


typedef struct stack_s {
	e_value	value;
	char	block;
	struct	stack_s *next;
} e_stack;


typedef struct label_s {
	int	label;
	struct	label_s *next;
} e_label;


typedef struct {
	char	type,
		*buffer,
		*bufPtr,
		*tmpSource,
		*names[20];
	int	fd,
		bufLen,
		bufRemain;
} e_sinfo;


typedef struct {
	e_stab	*literals[EMBER_HASH_LEN],
		*variables[EMBER_HASH_LEN];
	e_map	*macros[EMBER_HASH_LEN];
	e_funct	*scriptFuncts[EMBER_HASH_LEN],
		*moduleFuncts[EMBER_HASH_LEN],
		main;
	e_obj	*objectTable[EMBER_HASH_LEN];
	e_info	info;
	e_www	www;
	e_stack	*valueStack;
	e_fstck *functStack;
	e_label	*labelStack,
		*breakStack,
		*contStack;
	e_meth	methods;
	e_sinfo	source;
	e_value returnValue;
	int	errorCode,
		exitNow;
	char 	errorMsg[EMBER_ERR_LEN + 1],
		exitCode;
} ember;


typedef struct {
	char	magic[13],
		fileType,
		version;
} e_file;

typedef struct {
	char	type,
		length[4];
} e_bin;



extern	char	EMBER_PROG_NAME[],
		EMBER_PROG_VERSION[],
		EMBER_PROG_VENDOR[];


#define	E_FUNCT_ARGS(v)	e_stack	v[10]

/***********************************************************************
** Function Prototypes
*/


/* Symbol Table Routines */
int	eSymLookupVariable __ANSI_PROTO((ember*, char*));
int 	eSymCreateFunction __ANSI_PROTO((ember*, char*));
int 	eSymCreateLiteral __ANSI_PROTO((ember*,int, char*, int *));
int 	eSymAddFunctionArg __ANSI_PROTO((ember*, char*, int));
int 	eSymAddExternalFunction __ANSI_PROTO((ember*, e_extern*));
int 	eSymSetArrayElement __ANSI_PROTO((ember*,e_value*,e_value*,int));
int 	eSymSetAssocElement __ANSI_PROTO((ember*,e_value*,e_value*,char*));
int	eSymExpandLiteral __ANSI_PROTO((ember*, e_value*, e_value*));
int	eSymGetNumArrayElements __ANSI_PROTO((ember*, e_value*));
int	eSymGetNumAssocElements __ANSI_PROTO((ember*, e_value*));
int 	eSymAddVariableMap __ANSI_PROTO((ember*, char*, int));
void	eSymDupValue __ANSI_PROTO((ember*, e_value*, e_value*));
void	eSymDumpSymbols __ANSI_PROTO((ember*, e_stab**));
void	eSymFreeSymbols __ANSI_PROTO((ember*, e_stab**));
void	eSymFreeValue __ANSI_PROTO((ember*, e_value*));
void	eSymDumpSymbolMap __ANSI_PROTO((ember*, e_map**));
void	eSymFreeSymbolMap __ANSI_PROTO((ember*, e_map**));
void	eSymInitArray __ANSI_PROTO((ember*, e_value*));
void 	eSymFreeArgList __ANSI_PROTO((ember*, e_funct*));
e_map	*eSymFindVariableMap __ANSI_PROTO((ember*, e_map**, char*));
e_stab	*eSymGetVariableById __ANSI_PROTO((ember*, int));
e_stab 	*eSymGetVariableByName __ANSI_PROTO((ember*, char*));
e_stab	*eSymGetLiteral __ANSI_PROTO((ember*, int));
e_stab	*eSymCreateVariable __ANSI_PROTO((ember*, char*));
e_funct *eSymGetFunctionById __ANSI_PROTO((ember*, int));
e_funct *eSymGetFunctionByName __ANSI_PROTO((ember*, char*));
e_value *eSymGetArrayElement __ANSI_PROTO((ember*, e_value*, int));
char	*eSymGetVariableValue __ANSI_PROTO((ember*, char*));

int 	eSymSetIntValue __ANSI_PROTO((ember*, e_value*, int));
int 	eSymSetUintValue __ANSI_PROTO((ember*, e_value*, uint));
int 	eSymSetRealValue __ANSI_PROTO((ember*, e_value*, double));
int 	eSymSetTextValue __ANSI_PROTO((ember*, e_value*, char *));

int 	eSymCreateIntMacro __ANSI_PROTO((ember*, char*, int));
int 	eSymCreateTextMacro __ANSI_PROTO((ember*, char*, char *));
int 	eSymCreateRealMacro __ANSI_PROTO((ember*, char*, double));

/* Code Generation Routines */
int	eCodeAlu __ANSI_PROTO((ember*, int));
int	eCodeStore __ANSI_PROTO((ember*, int));
int	eCodeArrayStore __ANSI_PROTO((ember*, int));
int	eCodeAssocStore __ANSI_PROTO((ember*, int));
int 	eCodeTypeCast __ANSI_PROTO((ember*, int));
int	eCodePushLiteral __ANSI_PROTO((ember*, int, char *));
int 	eCodePushVariable __ANSI_PROTO((ember*, char*));
int	eCodePushNegativeLiteral __ANSI_PROTO((ember*, int, char *));
int	eCodeGetLabel __ANSI_PROTO((ember*));
int	eCodeJZ __ANSI_PROTO((ember*, int));
int	eCodePushLabel __ANSI_PROTO((ember*, int));
int	eCodePushMacro __ANSI_PROTO((ember*, char*));
int	eCodePushContLabel __ANSI_PROTO((ember*, int));
int	eCodePushBreakLabel __ANSI_PROTO((ember*, int));
int	eCodePopLabel __ANSI_PROTO((ember*));
int	eCodePopContLabel __ANSI_PROTO((ember*));
int	eCodePopBreakLabel __ANSI_PROTO((ember*));
int	eCodeJMP __ANSI_PROTO((ember*, int));
int	eCodeLabel __ANSI_PROTO((ember*, int));
int	eCodeJMPB __ANSI_PROTO((ember*, int));
int	eCodeCMP __ANSI_PROTO((ember*, int));
void	eCodeDumpCode __ANSI_PROTO((ember*, e_code*));
void	eCodeFreeCode __ANSI_PROTO((ember*, e_code*));
int	eCodeArrayDeref __ANSI_PROTO((ember*));
int	eCodeAssocDeref __ANSI_PROTO((ember*));
int	eCodeCount __ANSI_PROTO((ember*));
int	eCodeReturn __ANSI_PROTO((ember*));
int	eCodeBlockIn __ANSI_PROTO((ember*));
int	eCodeBlockOut __ANSI_PROTO((ember*));
int	eCodeCall __ANSI_PROTO((ember*, char*, int));
int 	eCodeExists __ANSI_PROTO ((ember*, char*));

/* Runtime Routines */
int	eRunScript __ANSI_PROTO((ember *));
int	eRunCode __ANSI_PROTO((ember *, e_funct*));
char	*eRunExpandText __ANSI_PROTO((ember*, char*));
void 	eRunSetError();
void 	ePushFrame __ANSI_PROTO((ember*, e_stack*));
e_stack	*ePopFrame __ANSI_PROTO((ember*));

/* Utility Routines */
ember	*eCreateScript __ANSI_PROTO(());
int	eInit __ANSI_PROTO(( ));
int	eInitMain __ANSI_PROTO((ember*));
int	eEndOfInput __ANSI_PROTO(( ));
int	eFileSource __ANSI_PROTO((ember*, char*));
int	eAddModule __ANSI_PROTO((ember*, e_extern*, void (*)()));
int	eGetLineNum __ANSI_PROTO((ember*));
int	eBufferSource __ANSI_PROTO((ember*, char*));
int	eGenHash __ANSI_PROTO((char*));
int	eParseScript __ANSI_PROTO((ember*));
int	eStoreObject __ANSI_PROTO((ember*,char*,char*,void*));
void	*eFetchObject __ANSI_PROTO((ember*,char*,char*,int));
void	*eDeleteObject __ANSI_PROTO((ember*,char*,char*,int));
void	eFreeScript __ANSI_PROTO((ember*));
void	eFreeStackFrames __ANSI_PROTO(());
void 	eUtilResetError __ANSI_PROTO((ember*));
char	*eGetErrorMsg __ANSI_PROTO((ember*));

/* Type Routines */
int	eTypeGetBaseType __ANSI_PROTO((int));
int	eTypeGetRepType __ANSI_PROTO((int));
int	eTypeCompareValues __ANSI_PROTO((ember*,int,e_value*,e_value*,int*));
int	eTypeCheckMathOp __ANSI_PROTO((ember*,int,e_value*,e_value*));
e_type	*eTypeGetType __ANSI_PROTO((int));

/* Binary handling routines */
int 	eLoadLibrary __ANSI_PROTO((ember*, char*));

/* WWW routines */
int 	eWwwDecode __ANSI_PROTO((char *, char *, int));
int 	eWwwSendHeaders __ANSI_PROTO((ember*));
int 	eWwwSendResponse __ANSI_PROTO((ember*, char*));
char 	*eWwwUnEscape __ANSI_PROTO((char *));
void 	eWwwAddHeader __ANSI_PROTO((ember*, char*));
void 	eWwwAuth __ANSI_PROTO((ember*, char*, char*, char*, char*));
void 	eWwwSendAuthHeader __ANSI_PROTO((ember*, char*, char*, char*));

/* Memory Routines */
#define eMemMalloc(s,l)	eMemRealMalloc(s,l,__FILE__, __LINE__)
#define eMemFree(s,p)	eMemRealFree(s,p,__FILE__, __LINE__)
#define eMemStrdup(s,p)	eMemRealStrdup(s,p,__FILE__, __LINE__)
void	*eMemRealMalloc __ANSI_PROTO((ember*, size_t, char*, int));
void	eMemRealFree __ANSI_PROTO((ember*, void*, char*, int));
char	*eMemRealStrdup __ANSI_PROTO((ember*, char*, char*, int));

/* Dynamic module Routines */
int 	eDynLoadModule __ANSI_PROTO((ember*, char*));



/***********************************************************************
** Standard header file footer.  
*/

#ifdef __cplusplus
	}
#endif /* __cplusplus */
#endif /* file inclusion */

