%{

/*
** Copyright (c) 2001  Hughes Technologies Pty Ltd.  All rights
** reserved.  
**
** Terms under which this software may be used or copied are
** provided in the  specific license associated with this product.
**
** Hughes Technologies disclaims all warranties with regard to this 
** software, including all implied warranties of merchantability and 
** fitness, in no event shall Hughes Technologies be liable for any 
** special, indirect or consequential damages or any damages whatsoever 
** resulting from loss of use, data or profits, whether in an action of 
** contract, negligence or other tortious action, arising out of or in 
** connection with the use or performance of this software.
**
**
** $Id: lexer.l,v 1.19 2002/06/04 06:37:23 bambi Exp $
**
*/

/*
** Module	: Ember : lexer
** Purpose	: Scanner for Ember's lexicons
** Exports	: 
** Depends Upon	: 
*/



/**************************************************************************
** STANDARD INCLUDES
**************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>

#include "config/config.h"

#ifdef HAVE_UNISTD_H
#  include <unistd.h>
#endif
#ifdef HAVE_STRING_H
#  include <string.h>
#endif
#ifdef HAVE_STRINGS_H
#  include <strings.h>
#endif


/**************************************************************************
** MODULE SPECIFIC INCLUDES
**************************************************************************/

#include "ember.h"
#ifdef DEBUG
#  include "parser_debug.h"
#else
#  include "parser.h"
#endif

/**************************************************************************
** GLOBAL VARIABLES
**************************************************************************/

#define		notInComment	!emberScript->info.inComment

#ifdef DEBUG
#  define 	TOK_RETURN(x)	if(notInComment) return((int)x)
   ember	tmpScript, *emberScript = &tmpScript;
   YYSTYPE	yylval;
#else
#  define 	TOK_RETURN(x)	if(notInComment) return(x)
   extern 	ember *emberScript;
   extern	YYSTYPE yylval;
#endif

/**************************************************************************
** PRIVATE ROUTINES
**************************************************************************/


#ifdef FLEX_SCANNER

#ifndef DEBUG
#  define 	YY_INPUT(buf,len,max_size) eLexerInput(buf,&len,max_size)
#endif

/*
** Input routines for FLEX
*/

void eLexerInput(buf,len,max)
	char	*buf;
	int	*len,
		max;
{								
	e_sinfo	*source;

	source = &emberScript->source;
	if (source->tmpSource)
	{
		*len = strlen(source->tmpSource);
		bcopy(source->tmpSource, buf, *len);
		source->tmpSource = NULL;
	}
	if (source->type == EMBER_FILE_INPUT) 
	{
		*len = read(source->fd,buf,max);
	}
	if (source->type == EMBER_BUFFER_INPUT) 
	{
		if (source->bufLen == 0)
		{
			source->bufLen = strlen(source->buffer);
			source->bufRemain = source->bufLen;
			source->bufPtr = source->buffer;
		}
		if (source->bufRemain == 0)
		{
			*len = 0;
			return;
		}
		if (max >= source->bufRemain)
		{
			*len = source->bufRemain;
			bcopy(source->bufPtr, buf, *len);
			source->bufRemain = 0;
			return;
		}
		*len = max;
		bcopy(source->bufPtr, buf, max);
		source->bufPtr += max;
		source->bufRemain -= max;
		return;
	}
}

#else /* FLEX_SCANNER */

/* Must be lex rather than flex */

#undef 	input
#define	input eLexerInput
#undef 	unput
#define	unput(c) eLexerUnput(c)

#define	lexMaxLen	1024
static 	char		lexBuf[lexMaxLen],
			lexUnBuf[lexMaxLen],
			*lexBufPtr = NULL,
			*lexUnBufPtr = NULL;
static	int		lexBufLen = 0;

void eLexerUnput(c)
	char	c;
{
	if (lexUnBufPtr == NULL)
		lexUnBufPtr = lexUnBuf;
	else
		lexUnBufPtr++;
	*lexUnBufPtr = c;
}

char eLexerInput()
{								
	e_sinfo	*source;
	char	c;

	/*
	** Check to see if there's any "unput" data hanging around
	*/
	if (lexUnBufPtr != NULL)
	{
		c = *lexUnBufPtr;
		if (lexUnBufPtr == lexUnBuf)
			lexUnBufPtr = NULL;
		else
			lexUnBufPtr--;
		return(c);
	}

	/*
	** Nope.  Lets get on with the show.  If are out of data then
	** get some more from the source.
	*/
	source = &emberScript->source;
	if (lexBufPtr == NULL)
	{
		if (source->type == EMBER_FILE_INPUT) 
		{
			lexBufLen = read(source->fd,lexBuf,lexMaxLen);
			lexBufPtr = lexBuf;
			if (lexBufLen == 0)
			{
				lexBufPtr = NULL;
				return(0);
			}
		}
		if (source->type == EMBER_BUFFER_INPUT) 
		{
			if (source->bufLen == 0)
			{
				source->bufLen = strlen(source->buffer);
				source->bufRemain = source->bufLen;
				source->bufPtr = source->buffer;
			}
			if (source->bufRemain == 0)
			{
				lexBufLen = 0;
				return(0);
			}
			if (lexMaxLen >= source->bufRemain)
			{
				lexBufLen = source->bufRemain;
				bcopy(source->bufPtr, lexBuf, lexBufLen);
				source->bufRemain = 0;
			}
			else
			{
				lexBufLen = lexMaxLen;
				bcopy(source->bufPtr, lexBuf, lexMaxLen);
				source->bufPtr += lexMaxLen;
				source->bufRemain -= lexMaxLen;
			}
			lexBufPtr = lexBuf;
		}
	}

	c = *lexBufPtr;
	lexBufPtr++;

	/*
	** If we are at the end of the buffer set the pointer to NULL
	** so that the next call will force a buffer read;
	*/
	if (lexBufPtr >= lexBuf + lexBufLen)
		lexBufPtr = NULL;

	return(c);
}


#endif /* FLEX_SCANNER */


void yyerror(msg)
	char	*msg;
{
        snprintf(emberScript->errorMsg, EMBER_ERR_LEN, "%s near [%s]",
                msg, yylval?yytext:"");
}


static void storeLexicon()
{
	if(notInComment)
		yylval = (char *)strdup(yytext);
	else
		yylval = yytext;
}

static void storeValue(val)
	char	*val;
{
	if(notInComment)
		yylval = (char *)strdup(val);
	else
		yylval = yytext;
}

static void storeToken(token)
	int	token;
{
	yylval = (char *)token;
}

static void startComment()
{
	emberScript->info.inComment = 1;
	emberScript->info.commentStart=emberScript->info.curLineNum;
}

static int endComment()
{
	if (emberScript->info.inComment == 0)
		return(-1);
	emberScript->info.inComment = 0;
	emberScript->info.commentStart = 0;
	return(0);
}

/**************************************************************************
** PUBLIC ROUTINES
**************************************************************************/


static	int	yyCondState=0;

#ifdef DEBUG
int main()
{
	char	*yylex();
	char	*p;
	bzero(&tmpScript, sizeof(tmpScript));
	tmpScript.info.lexStartState = -1;
	tmpScript.www.parseHtml = 0;
	while((p = yylex()))
	{
		if ((int)p < 127)
		{
			printf("Line %-3d  %c               is \"%s\"\n", 
				emberScript->info.curLineNum,
				(char)(int)p, yytext);
		}
		else
		{
			printf("Line %-3d  %-15.15s is \"%s\"\n", 
				emberScript->info.curLineNum,
				p, yytext);
		}
	}
	exit(0);
}
#endif


%}

alpha		[a-zA-Z_]
num		[0-9]
nonzero		[1-9]
hexnum		[0-9a-fA-F]
alphanum	[a-zA-Z_0-9\.]
text		[^"]
char		[^\n\r']
white		[ \t]
path		[a-zA-Z1-9_\./\-]
any		.

%x WWW
%x CODE

%%

%{
	if(emberScript->info.lexStartState == -1)
	{
		if (emberScript->www.parseHtml)
		{
			if (emberScript->www.parseHtml == 1)
			{
				emberScript->info.lexStartState = WWW;
				TOK_RETURN(HTML_START);
			}
			if (emberScript->www.parseHtml == 2)
			{
				emberScript->info.lexStartState = WWW;
				TOK_RETURN(HTML_END);
			}
		}
		else
		{
			emberScript->info.lexStartState = CODE;
		}
	}
	BEGIN emberScript->info.lexStartState;
%}


<WWW>[ \t]*<!			{ 
				  emberScript->info.lexStartState = CODE;
				  TOK_RETURN(HTML_END);
				}
<WWW>[^\n]*			{ storeLexicon(); TOK_RETURN(HTML);}
<WWW>\n			{ 
				  emberScript->info.curLineNum++;
				  storeLexicon(); TOK_RETURN(HTML);
				}

<CODE>">="			{storeToken(COND_GE); TOK_RETURN(COND_GE);}
<CODE>"<="			{storeToken(COND_LE); TOK_RETURN(COND_LE);}
<CODE>"!="			{storeToken(COND_NE); TOK_RETURN(COND_NE);}
<CODE>"<>"			{storeToken(COND_NE); TOK_RETURN(COND_NE);}
<CODE>"=="			{storeToken(COND_EQ); TOK_RETURN(COND_EQ);}
<CODE>">"			{ 
				  if (yyCondState)
				  {
				  	storeToken(COND_GT);
					TOK_RETURN(COND_GT);
				  }
				  else
				  {
					emberScript->info.lexStartState = WWW;
					TOK_RETURN(HTML_START);
				  }
			  	}
<CODE>"<"			{storeToken(COND_LT); TOK_RETURN(COND_LT);}
<CODE>"=~"			{storeToken(COND_RE); TOK_RETURN(COND_RE);}
<CODE>"&&"			{storeToken(COND_AND); TOK_RETURN(COND_AND);}
<CODE>"||"			{storeToken(COND_OR); TOK_RETURN(COND_OR);}

<CODE>"+="			{storeToken(OP_AUTOADD);TOK_RETURN(OP_AUTOADD);}
<CODE>"-="			{storeToken(OP_AUTOSUB);TOK_RETURN(OP_AUTOSUB);}
<CODE>"*="			{storeToken(OP_AUTOMUL);TOK_RETURN(OP_AUTOMUL);}
<CODE>"/="			{storeToken(OP_AUTODIV);TOK_RETURN(OP_AUTODIV);}
<CODE>"++"			{storeToken(OP_INC); TOK_RETURN(OP_INC);}
<CODE>"--"			{storeToken(OP_DEC); TOK_RETURN(OP_DEC);}
<CODE>"<<"			{storeToken(OP_SHIFT_L);TOK_RETURN(OP_SHIFT_L);}
<CODE>">>"			{storeToken(OP_SHIFT_R);TOK_RETURN(OP_SHIFT_R);}

<CODE>"+"			{storeToken(OP_ADD); TOK_RETURN(OP_ADD);}
<CODE>"-"			{storeToken(OP_SUB); TOK_RETURN(OP_SUB);}
<CODE>"*"			{storeToken(OP_MUL); TOK_RETURN(OP_MUL);}
<CODE>"/"			{storeToken(OP_DIV); TOK_RETURN(OP_DIV);}
<CODE>"%"			{storeToken(OP_MOD); TOK_RETURN(OP_MOD);}
<CODE>"&"			{storeToken(OP_AND); TOK_RETURN(OP_AND);}
<CODE>"|"			{storeToken(OP_OR); TOK_RETURN(OP_OR);}

<CODE>"#!"{white}*{path}+	{storeToken(EXEC_PATH); TOK_RETURN(EXEC_PATH);}
<CODE>"#"			{storeToken(OP_OR); TOK_RETURN(COUNT);}
<CODE>"if"			{storeToken(IF); TOK_RETURN(IF);}
<CODE>"else"			{storeToken(ELSE); TOK_RETURN(ELSE);}
<CODE>"while"			{storeToken(WHILE); TOK_RETURN(WHILE);}
<CODE>"break"			{storeToken(BREAK); TOK_RETURN(BREAK);}
<CODE>"continue"		{storeToken(CONTINUE); TOK_RETURN(CONTINUE);}
<CODE>"return"			{storeToken(RETURN); TOK_RETURN(RETURN);}
<CODE>"funct"			{storeToken(FUNCT); TOK_RETURN(FUNCT);}
<CODE>"load"			{storeToken(LOAD); TOK_RETURN(LOAD);}
<CODE>"modload"			{storeToken(MOD_LOAD); TOK_RETURN(MOD_LOAD);}
<CODE>"exists"			{storeToken(EXISTS); TOK_RETURN(EXISTS);}

<CODE>"int"			{storeToken(T_INT); TOK_RETURN(T_INT);}
<CODE>"uint"			{storeToken(T_UINT); TOK_RETURN(T_UINT);}
<CODE>"text"			{storeToken(T_TEXT); TOK_RETURN(T_TEXT);}
<CODE>"char"			{storeToken(T_CHAR); TOK_RETURN(T_CHAR);}
<CODE>"real"			{storeToken(T_REAL); TOK_RETURN(T_REAL);}
<CODE>"money"			{storeToken(T_MONEY); TOK_RETURN(T_MONEY);}
<CODE>"array"			{storeToken(T_ARRAY);TOK_RETURN(T_ARRAY);}

<CODE>"bool"			{storeToken(T_INT); TOK_RETURN(T_INT);}
<CODE>"true"			{storeValue("1"); TOK_RETURN(LIT_INT);}
<CODE>"True"			{storeValue("1"); TOK_RETURN(LIT_INT);}
<CODE>"false"			{storeValue("0"); TOK_RETURN(LIT_INT);}
<CODE>"False"			{storeValue("0"); TOK_RETURN(LIT_INT);}

<CODE>"/*"			{startComment(); }
<CODE>"*/"			{if(endComment() < 0) TOK_RETURN(BAD_LEX); }

<CODE>"\""({text}|"\\\"")*"\""	{
					char	*cp;

					storeLexicon(); 
					cp = yylval;
					while(*cp)
					{
					    if (*cp == '\n')
						emberScript->info.curLineNum++;
					    cp++;
					}
					TOK_RETURN(LIT_TEXT);
				}

<CODE>{num}+"."{num}+		{storeLexicon(); TOK_RETURN(LIT_REAL);}
<CODE>{num}+			{storeLexicon(); TOK_RETURN(LIT_INT);}
<CODE>"0x"{hexnum}*		{storeLexicon(); TOK_RETURN(LIT_INT);}
<CODE>"-"{nonzero}{num}+"."{num}+ {storeLexicon(); TOK_RETURN(LIT_REAL);}
<CODE>"-0."{num}+		{storeLexicon(); TOK_RETURN(LIT_REAL);}
<CODE>"-"{num}+			{storeLexicon(); TOK_RETURN(LIT_NEG_INT);}

<CODE>${alpha}{alphanum}*	{storeLexicon(); TOK_RETURN(VAR);}
<CODE>@{alpha}{alphanum}*	{storeLexicon(); TOK_RETURN(VAR);}
<CODE>{alpha}{alphanum}*	{storeLexicon(); TOK_RETURN(IDENT);}

<CODE>"{"			{storeToken(L_BRACE); TOK_RETURN(L_BRACE);}
<CODE>"}"			{storeToken(R_BRACE); TOK_RETURN(R_BRACE);}
<CODE>"("			{ 
				  yyCondState++;
			 	  storeToken(L_BRACKET); TOK_RETURN(L_BRACKET);
				}
<CODE>")"			{ 
				  yyCondState--;
				  storeToken(R_BRACKET); TOK_RETURN(R_BRACKET);
				}
<CODE>"["			{storeToken(L_SQUARE); TOK_RETURN(L_SQUARE);}
<CODE>"]"			{storeToken(R_SQUARE); TOK_RETURN(R_SQUARE);}
<CODE>"!"			{storeToken(BANG); TOK_RETURN(BANG);}
<CODE>";"			{storeToken(SEMI_COLON);TOK_RETURN(SEMI_COLON);}
<CODE>","			{storeToken(COMMA); TOK_RETURN(COMMA);}
<CODE>"="			{storeToken(ASSIGN); TOK_RETURN(ASSIGN);}

<CODE>"\n"			{emberScript->info.curLineNum++; }
<CODE>{white}*			;
<CODE>{any}			{storeLexicon(); TOK_RETURN(yytext[0]);}

%%

/*
** yywarp - dummy routine for flex
**
** We put it here to get it past the prototype generated by flex.
*/
int yywrap()
{
	/* 
	** Flex expects a function called yywrap to exist.  It is called
	** at the end of input.  This allows the scanner's caller to
	** switch buffers and continue the scan if required.  We don't
	** want that behavious so we just define a dummy function that
	** returns 1 (do not continue scanning) to the scanner
	*/
	
	if (emberScript->www.parseHtml == 1)
	{
		emberScript->info.lexStartState = -1;
		emberScript->www.parseHtml = 2;
		return(0);
	}
	return(1);
}

