/*
** Copyright (c) 2001  Hughes Technologies Pty Ltd.  All rights
** reserved.  Terms under which this software may be used or copied are
** provided in the  specific license associated with this product.
**
** Hughes Technologies disclaims all warranties with regard to this 
** software, including all implied warranties of merchantability and 
** fitness, in no event shall Hughes Technologies be liable for any 
** special, indirect or consequential damages or any damages whatsoever 
** resulting from loss of use, data or profits, whether in an action of 
** contract, negligence or other tortious action, arising out of or in 
** connection with the use or performance of this software.
**
**
** $Id: mod_net.c,v 1.7 2002/05/31 04:21:29 bambi Exp $
**
*/

/*
** Module	: Network Library Module
** Purpose	: 
** Exports	: 
** Depends Upon	: 
*/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <netdb.h>
#include <string.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>



#include <config/config.h>

#ifdef HAVE_STRINGS_H
#  include <strings.h>
#endif

#include "ember.h"
#include "parser.h"



/**************************************************************************
** GLOBAL VARIABLES
**************************************************************************/


/**************************************************************************
** PRIVATE ROUTINES
**************************************************************************/

static int scanCidr(val, result, length)
	char	*val;
	u_int	*result,
		*length;
{
	u_int	res, res1, res2, res3, res4, res5;
	char	*cp;

	cp = val;
	res1 = atoi(cp);
	cp = index(cp,'.');
	if (!cp)
		return(-1);
	cp++;
	res2 = atoi(cp);
	cp = index(cp,'.');
	if (!cp)
		return(-1);
	cp++;
	res3 = atoi(cp);
	cp = index(cp,'.');
	if (!cp)
		return(-1);
	cp++;
	res4 = atoi(cp);
	cp = index(cp,'/');
	if (!cp)
	{
		res5 = 32;
	}
	else
	{
		cp++;
		res5 = atoi(cp);
	}

	if (res1>255 || res2>255 || res3>255 || res4>255 || res5>32)
	{
		return(-1);
	}
	res = (res1 << 24) + (res2 << 16) + (res3 << 8) + res4;
	*result = res;
	*length = res5;
	return(0);
}


/**************************************************************************
** PUBLIC ROUTINES
**************************************************************************/

/*! file
!name mod_net
!brief Ember module providing network related routines
!descr The network module provides access to library functions related
!descr to performing network related functions.  These operations
!descr include communications over the network (using TCP/IP via sockets)
!descr and access to the network name servers (DNS).
!*/



/***************************************************************************/
/*! function
!name gethostbyname( )
!brief Lookup the IP address of a host
!usage gethostbyname($hostname)
!param text $hostname (The hostname to be resolved)
!return text (IP address.  Empty string with $ERRMSG set on error)
!descr This routine provides an interface to the network name server
!descr (the DNS).  When passed a network hostname, the DNS is queried and the
!descr IP address of the host is determined.
!example $addr = gethostbyname("localhost");
!example if (# $addr == 0)
!example {
!example	echo("Hostname lookup failed : $ERRMSG\n");
!example }
!*/
int doGetHostByName(script, args)
        ember	*script;
	e_stack	*args;
{
        char    *hostname;
        struct  hostent *hp;

        hostname = args->value.value.textVal;
        hp = gethostbyname(hostname);
        if (!hp)
        {
		eRunSetError(script, EMBER_TYPE_TEXT, "Host unknown '%s'", 
			hostname);
                return(0);
        }
	eSymSetTextValue(script,&script->returnValue, 
		inet_ntoa(*(struct in_addr*)*(hp->h_addr_list)));
	return(0);
}


/***************************************************************************/
/*! function
!name gethostbyaddr( )
!brief Lookup the hostname assocaited with an address
!usage gethostbyaddr($addr)
!param text $addr (The IP address to be resolved)
!return text (Hostname.  Empty string with $ERRMSG set on error)
!descr This routine provides an interface to the network name server
!descr (the DNS).  When passed an IP address , the DNS is queried and the
!descr hostname associated with that IP address is returned.
!example $hostname = gethostbyaddr("127.0.0.1");
!example if (# $hostname == 0)
!example {
!example	echo("Address lookup failed : $ERRMSG\n");
!example }
!*/

#ifndef INADDR_NONE
#  define INADDR_NONE -1
#endif

int doGetHostByAddr(script, args)
	ember	*script;
	e_stack	*args;
{
        char    *hostAddr;
        struct  hostent *hp;
        struct  sockaddr_in addr;
        u_long  laddr;

        hostAddr = emTextArg(args);
        (void)bzero(&addr, sizeof(addr));
        addr.sin_family = AF_INET;
        laddr = inet_addr(hostAddr);
        if (laddr == INADDR_NONE)
        {
		eRunSetError(script,EMBER_TYPE_TEXT,"Bad IP Address '%s'",
			hostAddr);
                return(0);
        }
        addr.sin_addr.s_addr = laddr;
        hp = gethostbyaddr((char *)&(addr.sin_addr),sizeof(addr.sin_addr),
                AF_INET);
        if (!hp)
        {
		eRunSetError(script,EMBER_TYPE_TEXT, "Host unknown '%s'",
			hostAddr);
                return(0);
                return(-1);
        }
	eSymSetTextValue(script,&script->returnValue, hp->h_name);
	return(0);
}






int doIsInCidrBlock(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*arg1,
		*arg2;
	int	addr1, len1,
		addr2, len2,
		count,
		mask;

        arg1 = emTextArg(args);
        arg2 = emTextArg(args->next);
	if(scanCidr(arg1, &addr1, &len1) < 0)
	{
		eRunSetError(script,EMBER_TYPE_INT, 
			"Invalid Address Format '%s'", arg1);
		return(0);
	}
	if(scanCidr(arg2, &addr2, &len2) < 0)
	{
		eRunSetError(script,EMBER_TYPE_INT, 
			"Invalid Address Format '%s'", arg2);
		return(0);
	}
	if(len2 < len1)
	{
		eRunSetError(script,EMBER_TYPE_INT, 
			"'%s' must be more specific than '%s'",
			arg1, arg2);
		return(0);
	}

	mask = count = 0;
	while(count < len1)
	{
		mask = (mask << 1) + 1;
		count++;
	}
	mask = mask << (32 - len1);
	if ( (addr1 & mask) == (addr2 & mask))
	{
		eSymSetIntValue(script,&script->returnValue, 1);
	}
	else
	{
		eSymSetIntValue(script,&script->returnValue, 0);
	}
	return(0);
}
