/*
** Copyright (c) 2001  Hughes Technologies Pty Ltd.  All rights
** reserved.  Terms under which this software may be used or copied are
** provided in the  specific license associated with this product.
**
** Hughes Technologies disclaims all warranties with regard to this 
** software, including all implied warranties of merchantability and 
** fitness, in no event shall Hughes Technologies be liable for any 
** special, indirect or consequential damages or any damages whatsoever 
** resulting from loss of use, data or profits, whether in an action of 
** contract, negligence or other tortious action, arising out of or in 
** connection with the use or performance of this software.
**
**
** $Id: mod_syscall.c,v 1.16 2002/07/17 01:32:53 bambi Exp $
**
*/

/*
** Module	: System call Library Module
** Purpose	: 
** Exports	: 
** Depends Upon	: 
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <errno.h>
#include <time.h>
#include <signal.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <pwd.h>
#include <config/config.h>

#ifdef HAVE_STRINGS_H
#  include <strings.h>
#endif

#include "ember.h"
#include "parser.h"



/**************************************************************************
** GLOBAL VARIABLES
**************************************************************************/


/**************************************************************************
** PRIVATE ROUTINES
**************************************************************************/


/**************************************************************************
** PUBLIC ROUTINES
**************************************************************************/


int doExit(script, args)
	ember	*script;
	e_stack	*args;
{
	int	val;

	val = emIntArg(args);
	script->exitCode = val;
	script->exitNow = 1;
	return(0);
}


int doUnlink(script, args)
	ember	*script;
	e_stack	*args;
{
	int 	res;

	res = unlink(emTextArg(args));
	if (res < 0)
		eRunSetError(script,EMBER_TYPE_INT, strerror(errno));
       	eSymSetIntValue(script,&script->returnValue, res);
	return(0);
}




int doChmod(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*path;
	int 	mode,
		res;

	mode = emIntArg(args);
	path = emTextArg(args->next);

	res = chmod(path,mode);
	if (res < 0)
		eRunSetError(script,EMBER_TYPE_INT, strerror(errno));
       	eSymSetIntValue(script,&script->returnValue, res);
	return(0);
}



int doChown(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*path,
		*uname;
	int 	mode,
		res,
		uid;
	struct 	passwd *pw;

	mode = emIntArg(args);
	uname = emTextArg(args->next);
	pw = getpwnam(uname);
	if (pw == NULL)
	{
		eRunSetError(script,EMBER_TYPE_INT, "Invalud username");
		return(0);
	}
	uid = pw->pw_uid;
	res = chown(path,uid,-1);
	if (res < 0)
		eRunSetError(script,EMBER_TYPE_INT, strerror(errno));
       	eSymSetIntValue(script,&script->returnValue, res);
	return(0);
}




int doSleep(script, args)
	ember	*script;
	e_stack	*args;
{
	int 	time;

	time = emIntArg(args);
	sleep(time);
	return(0);
}




int doSystem(script, args)
	ember	*script;
	e_stack	*args;
{
	char 	*command;
	int	res,
		pid,
		status;

	command = emTextArg(args);
	/*
	** Don't use system.  Do it ourselves so that we can be sure
	** of the exit value.  Some systems stuff this up
	res = system(command);
	*/

	pid = fork();
	if (pid < 0)
	{
		res = -1;
	}
	if (pid == 0)
	{
                /* child */
                execl("/bin/sh", "sh", "-c", command, NULL);
                exit(-1);
        }
        wait(&status);
        if (WIFEXITED(status))
        {
                res = WEXITSTATUS(status);
        }
        else
        {
		res = -1;
	}
	if (res < 0)
		eRunSetError(script,EMBER_TYPE_INT, strerror(errno));
       	eSymSetIntValue(script,&script->returnValue, res);
	return(0);
}


int doUmask(script, args)
	ember	*script;
	e_stack	*args;
{
	int	mask;

	mask = emIntArg(args);
	mask = umask(mask);
       	eSymSetIntValue(script,&script->returnValue, mask);
	return(0);

}

int doGetpid(script, args)
	ember	*script;
	e_stack	*args;
{
	int	pid;

	pid = getpid();
       	eSymSetIntValue(script,&script->returnValue, pid);
	return(0);
}

int doGetppid(script, args)
	ember	*script;
	e_stack	*args;
{
	int	pid;

	pid = getppid();
       	eSymSetIntValue(script,&script->returnValue, pid);
	return(0);
}


int doGetCwd(script, args)
	ember	*script;
	e_stack	*args;
{
	char	path[255];

	if(getcwd(path,255) == NULL)
		eRunSetError(script,EMBER_TYPE_TEXT, strerror(errno));
       	eSymSetTextValue(script,&script->returnValue, path);
	return(0);
}


int doGetEnv(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*varName,
		*value;

	varName = emTextArg(args);
	value = getenv(varName);
	if (value)
       		eSymSetTextValue(script,&script->returnValue, value);
	else
       		eSymSetTextValue(script,&script->returnValue, "");
	return(0);
}


int doChdir(script, args)
	ember	*script;
	e_stack	*args;
{
	int	res;
	char	*path;

	path = emTextArg(args);
	res = chdir(path);
	if (res < 0)
		eRunSetError(script,EMBER_TYPE_INT, strerror(errno));
       	eSymSetIntValue(script,&script->returnValue, res);
	return(0);
}

int doMkdir(script, args)
	ember	*script;
	e_stack	*args;
{
	int	res;
	char	*path;

	path = emTextArg(args);
	res  = mkdir(path,0777);
	if (res < 0)
		eRunSetError(script,EMBER_TYPE_INT, strerror(errno));
       	eSymSetIntValue(script,&script->returnValue, res);
	return(0);
}


int doRmdir(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*path;
	int	res;

	path = emTextArg(args);
	res = rmdir(path);
	if (res < 0)
		eRunSetError(script,EMBER_TYPE_INT, strerror(errno));
       	eSymSetIntValue(script,&script->returnValue, res);
	return(0);
}


int doRename(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*old,
		*new;
	int	res;

	old = emTextArg(args);
	new = emTextArg(args->next);
	res = rename(old,new);
	if (res < 0)
		eRunSetError(script,EMBER_TYPE_INT, strerror(errno));
       	eSymSetIntValue(script,&script->returnValue, res);
	return(0);
}


int doTruncate(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*path;
	int	length,
		res;

	path = emTextArg(args);
	length = emIntArg(args->next);
	res = truncate(path,length);
	if (res < 0)
		eRunSetError(script,EMBER_TYPE_INT, strerror(errno));
       	eSymSetIntValue(script,&script->returnValue, res);
	return(0);
}


int doLink(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*path,
		*new;
	int	res;

	path = emTextArg(args);
	new = emTextArg(args->next);

#if	! defined(HAVE_LINK)
	strcpy(script->errorMsg, "Link not supported");
        return(-1);
#endif

	res = link(path,new);
	if (res < 0)
		eRunSetError(script,EMBER_TYPE_INT, strerror(errno));
       	eSymSetIntValue(script,&script->returnValue, res);
	return(0);
}


int doSymlink(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*path,
		*new;
	int	res;

	path = emTextArg(args);
	new = emTextArg(args->next);
#if	! defined(HAVE_SYMLINK)
	strcpy(script->errorMsg, "Symbolic link not supported");
        return(-1);
#endif

	res = symlink(path,new);
	if (res < 0)
		eRunSetError(script,EMBER_TYPE_INT, strerror(errno));
       	eSymSetIntValue(script,&script->returnValue, res);
	return(0);
}



int doTest(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*filename,
		test;
	struct	stat buf;
	int 	result = 0;

	test = *(emTextArg(args));
	filename = emTextArg(args->next);
	if (stat(filename,&buf) < 0)
		result = 0;
	else
	{
		switch(test)
		{
			case 'b':
				result = S_ISBLK(buf.st_mode);
				break;
			case 'c':
				result = S_ISCHR(buf.st_mode);
				break;
			case 'd':
				result = S_ISDIR(buf.st_mode);
				break;
			case 'f':
				result = S_ISREG(buf.st_mode);
				break;
			case 'p':
				result = S_ISFIFO(buf.st_mode);
				break;
			case 's':
				result = S_ISREG(buf.st_mode) &&
					 (buf.st_size > 0);
				break;
			case 'u':
				result = buf.st_mode & S_ISUID;
				break;
			case 'g':
				result = buf.st_mode & S_ISGID;
				break;
			default:
				eRunSetError(script,EMBER_TYPE_INT,
					"Bad test type passed to test()");
				return(0);
		}
	}
       	eSymSetIntValue(script,&script->returnValue, result);
	return(0);
}



int doKill(script, args)
	ember	*script;
	e_stack	*args;
{
	int	sig,
		pid,
		res;

	pid = emIntArg(args);
	sig = emIntArg(args->next);
	if (sig < 0 || sig > NSIG-1)
	{
		eRunSetError(script,EMBER_TYPE_INT, "Bad Signal Number");
		return(0);
	}
	res = kill(pid,sig);
	if (res < 0)
	{
		eRunSetError(script,EMBER_TYPE_INT, strerror(errno));
		return(0);
	}
	eSymSetIntValue(script,&script->returnValue, 0);
	return(0);
}



int doStat(script, args)
	ember	*script;
	e_stack	*args;
{
	struct	stat buf;
	char	*path;
	int	index;
	e_value	val;

	path = emTextArg(args);
	if (stat(path,&buf) < 0)
	{
		eRunSetError(script,EMBER_TYPE_TEXT, strerror(errno));
		return(0);
	}

	index = 0;
	eSymSetIntValue(script,&val,buf.st_ino);
	eSymSetArrayElement(script, &script->returnValue, &val, index++);
	eSymSetIntValue(script,&val,buf.st_mode & 1023);
	eSymSetArrayElement(script, &script->returnValue, &val, index++);
	eSymSetIntValue(script,&val,buf.st_nlink);
	eSymSetArrayElement(script, &script->returnValue, &val, index++);
	eSymSetIntValue(script,&val,buf.st_uid);
	eSymSetArrayElement(script, &script->returnValue, &val, index++);
	eSymSetIntValue(script,&val,buf.st_gid);
	eSymSetArrayElement(script, &script->returnValue, &val, index++);
	eSymSetIntValue(script,&val,buf.st_size);
	eSymSetArrayElement(script, &script->returnValue, &val, index++);
	eSymSetIntValue(script,&val,buf.st_atime);
	eSymSetArrayElement(script, &script->returnValue, &val, index++);
	eSymSetIntValue(script,&val,buf.st_mtime);
	eSymSetArrayElement(script, &script->returnValue, &val, index++);
	eSymSetIntValue(script,&val,buf.st_ctime);
	eSymSetArrayElement(script, &script->returnValue, &val, index++);
	eSymSetIntValue(script,&val,buf.st_blksize);
	eSymSetArrayElement(script, &script->returnValue, &val, index++);
	eSymSetIntValue(script,&val,buf.st_blocks);
	eSymSetArrayElement(script, &script->returnValue, &val, index++);
	return(0);
}


int doTime(script, args)
	ember	*script;
	e_stack	*args;
{
	time_t	res;

	time(&res);
	if (res < 0)
		eRunSetError(script,EMBER_TYPE_TEXT, strerror(errno));
	eSymSetIntValue(script,&script->returnValue, res);
	return(0);
}


int doGetTzOffset(script, args)
	ember	*script;
	e_stack	*args;
{
	/*
	** Note : ftime etc return the GMT offset in minutes _west_ of GMT.
	** In reality, global time is shown in minutes _east_ of GMT.
	** This like a very US centric way of handling offsets.  We
	** correct for this and return it as minutes east
	*/
	time_t	day1,
		val;

	day1 = 60*60*24 ;
	val = day1 - mktime(gmtime(&day1));
	val /= 60;
	eSymSetIntValue(script,&script->returnValue, val);
	return(0);
}


int doSrandom(script, args)
	ember	*script;
	e_stack	*args;
{
	u_int	seed;

        seed = emIntArg(args);
	srandom(seed);
	return(0);
}




int doRandom(script, args)
	ember	*script;
	e_stack	*args;
{
	int	res;

	res = random();
	eSymSetIntValue(script,&script->returnValue, res);
	return(0);
}




int doCTime(script, args)
	ember	*script;
	e_stack	*args;
{
	time_t	time;
	char	*res;

        time = (time_t) emIntArg(args);
	res = (char *)ctime(&time);
	if (res == NULL)
	{
		eRunSetError(script,EMBER_TYPE_TEXT, strerror(errno));
	}
	else
	{
		eSymSetTextValue(script,&script->returnValue, res);
	}
	return(0);
}



int doGetUid(script, args)
	ember	*script;
	e_stack	*args;
{
	int	uid;

	uid = getuid();
	eSymSetIntValue(script,&script->returnValue, uid);
	return(0);
}


int doGetGid(script, args)
	ember	*script;
	e_stack	*args;
{
	int	gid;

	gid = getgid();
	eSymSetIntValue(script,&script->returnValue, gid);
	return(0);
}


int doGetEuid(script, args)
	ember	*script;
	e_stack	*args;
{
	int	uid;

	uid = geteuid();
	eSymSetIntValue(script,&script->returnValue, uid);
	return(0);
}


int doGetPwNam(script, args)
	ember	*script;
	e_stack	*args;
{
	struct 	passwd *pwe;
	char	*uname;
	e_value	val;
	int	index;

	uname = emTextArg(args);
	pwe = getpwnam(uname);
	if (!pwe)
	{
		eRunSetError(script, EMBER_TYPE_TEXT, "Unknown username");
		return(0);
	}
	index=0;
	eSymSetTextValue(script,&val,pwe->pw_name);
	eSymSetArrayElement(script, &script->returnValue, &val, index++);
	eSymSetTextValue(script,&val,pwe->pw_passwd);
	eSymSetArrayElement(script, &script->returnValue, &val, index++);
	eSymSetIntValue(script,&val,pwe->pw_uid);
	eSymSetArrayElement(script, &script->returnValue, &val, index++);
	eSymSetIntValue(script,&val,pwe->pw_gid);
	eSymSetArrayElement(script, &script->returnValue, &val, index++);
	eSymSetTextValue(script,&val,pwe->pw_gecos);
	eSymSetArrayElement(script, &script->returnValue, &val, index++);
	eSymSetTextValue(script,&val,pwe->pw_dir);
	eSymSetArrayElement(script, &script->returnValue, &val, index++);
	eSymSetTextValue(script,&val,pwe->pw_shell);
	eSymSetArrayElement(script, &script->returnValue, &val, index++);
	return(0);
}

	
int doGetPwUid(script, args)
	ember	*script;
	e_stack	*args;
{
	struct 	passwd *pwe;
	e_value	val;
	int	index,
		uid;

	uid = emIntArg(args);
	pwe = getpwuid(uid);
	if (!pwe)
	{
		eRunSetError(script, EMBER_TYPE_TEXT, "Unknown user id");
		return(0);
	}
	index=0;
	eSymSetTextValue(script,&val,pwe->pw_name);
	eSymSetArrayElement(script, &script->returnValue, &val, index++);
	eSymSetTextValue(script,&val,pwe->pw_passwd);
	eSymSetArrayElement(script, &script->returnValue, &val, index++);
	eSymSetIntValue(script,&val,pwe->pw_uid);
	eSymSetArrayElement(script, &script->returnValue, &val, index++);
	eSymSetIntValue(script,&val,pwe->pw_gid);
	eSymSetArrayElement(script, &script->returnValue, &val, index++);
	eSymSetTextValue(script,&val,pwe->pw_gecos);
	eSymSetArrayElement(script, &script->returnValue, &val, index++);
	eSymSetTextValue(script,&val,pwe->pw_dir);
	eSymSetArrayElement(script, &script->returnValue, &val, index++);
	eSymSetTextValue(script,&val,pwe->pw_shell);
	eSymSetArrayElement(script, &script->returnValue, &val, index++);
	return(0);
}

	


#ifdef NOTDEF
void doCreateVariable(params)
	plist_t	*params;
{
	char	*name,
		*value;
	sym_t	*var;

	name = params->sym->val;
	value = params->next->sym->val;

	if (symGetSymbol(name) != NULL)
	{
		runError("Variable already exists");
	}
	var = symCreateSymbol(name, EMBER_TYPE_CHAR, SCALAR);
	symStoreValue(var,value,strlen(value),EMBER_TYPE_CHAR);
	var->length = strlen(value);
}

void doTypeOf(params)
	plist_t	*params;
{

	if (!params)
	{
		runError("Too few parameters to typeof()");
	}
	if (params->next)
	{
		runError("Too many parameters to typeof()");
	}
	if (params->sym->array == ARRAY)
	{
		externReturn = createCharSymbol("array");
	}
	else
	{
		switch(params->sym->type)
		{
			case EMBER_TYPE_CHAR:
				externReturn = createCharSymbol("char");
				break;
			case EMBER_TYPE_INT:
				externReturn = createCharSymbol("int");
				break;
			case EMBER_TYPE_REAL:
				externReturn = createCharSymbol("real");
				break;
			case EMBER_TYPE_UINT:
				externReturn = createCharSymbol("uint");
				break;
			default:
				externReturn = createCharSymbol("unknown");
				break;
		}
	}
	return;
}
#endif
