/*
** Copyright (c) 2001  Hughes Technologies Pty Ltd.  All rights
** reserved.  Terms under which this software may be used or copied are
** provided in the  specific license associated with this product.
**
** Hughes Technologies disclaims all warranties with regard to this 
** software, including all implied warranties of merchantability and 
** fitness, in no event shall Hughes Technologies be liable for any 
** special, indirect or consequential damages or any damages whatsoever 
** resulting from loss of use, data or profits, whether in an action of 
** contract, negligence or other tortious action, arising out of or in 
** connection with the use or performance of this software.
**
**
** $Id: mod_time.c,v 1.4 2001/07/26 01:36:25 bambi Exp $
**
*/

/*
** Module	: Standard Library Module
** Purpose	: 
** Exports	: 
** Depends Upon	: 
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <sys/types.h>
#include <time.h>

#include <config/config.h>

#ifdef HAVE_STRINGS_H
#  include <strings.h>
#endif

#include "ember.h"



/**************************************************************************
** GLOBAL VARIABLES
**************************************************************************/


/**************************************************************************
** PRIVATE ROUTINES
**************************************************************************/


/**************************************************************************
** PUBLIC ROUTINES
**************************************************************************/


int doTime2UnixTime(script, args)
	ember	*script;
	e_stack	*args;
{
	int	sec, min, hour,
		day, mon, year;
	time_t	res;
	struct	tm timeVal;
#ifdef	HAVE_ISDST
	struct	tm *timeValLocal;
#endif

        sec = emIntArg(args);
        min = emIntArg(args->next);
        hour = emIntArg(args->next->next);
        day = emIntArg(args->next->next->next);
        mon = emIntArg(args->next->next->next->next);
        year = emIntArg(args->next->next->next->next->next);

	bzero(&timeVal, sizeof(timeVal));
#ifdef HAVE_ISDST
	res = time(NULL);
	timeValLocal = localtime(&res);
	timeVal.tm_isdst = timeValLocal->tm_isdst;
#endif
	timeVal.tm_sec = sec;
	timeVal.tm_min = min;
	timeVal.tm_hour = hour;
	timeVal.tm_mday = day;
	timeVal.tm_mon = mon - 1;
	timeVal.tm_year = year - 1900;
	res = mktime(&timeVal);
        eSymSetIntValue(script,&script->returnValue, res);
	return(0);
}


int doUnixTime2Year(script, args)
	ember	*script;
	e_stack	*args;
{
	int	year;
	char	buf[10];
	struct	tm *locTime;
	time_t 	time;

        time = emIntArg(args);
	locTime = localtime(&time);
	strftime(buf,sizeof(buf),"%Y",locTime);
	year = atoi(buf);
        eSymSetIntValue(script,&script->returnValue, year);
	return(0);
}



int doUnixTime2Month(script, args)
	ember	*script;
	e_stack	*args;
{
	int	month;
	char	buf[10];
	struct	tm *locTime;
	time_t 	time;

        time = (time_t) emIntArg(args);
	locTime = localtime(&time);
	strftime(buf,sizeof(buf),"%m",locTime);
	month = atoi(buf);
        eSymSetIntValue(script,&script->returnValue, month);
	return(0);
}


int doUnixTime2Day(script, args)
	ember	*script;
	e_stack	*args;
{
	int	day;
	char	buf[10];
	struct	tm *locTime;
	time_t 	time;

        time = (time_t) emIntArg(args);
	locTime = localtime(&time);
	strftime(buf,sizeof(buf),"%d",locTime);
	day = atoi(buf);
        eSymSetIntValue(script,&script->returnValue, day);
	return(0);
}


int doUnixTime2Hour(script, args)
	ember	*script;
	e_stack	*args;
{
	int	hour;
	char	buf[10];
	struct	tm *locTime;
	time_t 	time;

        time = (time_t) emIntArg(args);
	locTime = localtime(&time);
	strftime(buf,sizeof(buf),"%H",locTime);
	hour = atoi(buf);
        eSymSetIntValue(script,&script->returnValue, hour);
	return(0);
}


int doUnixTime2Min(script, args)
	ember	*script;
	e_stack	*args;
{
	int	min;
	char	buf[10];
	struct	tm *locTime;
	time_t 	time;

        time = (time_t) emIntArg(args);
	locTime = localtime(&time);
	strftime(buf,sizeof(buf),"%M",locTime);
	min = atoi(buf);
        eSymSetIntValue(script,&script->returnValue, min);
	return(0);
}


int doUnixTime2Sec(script, args)
	ember	*script;
	e_stack	*args;
{
	int	sec;
	char	buf[10];
	struct	tm *locTime;
	time_t 	time;

        time = (time_t) emIntArg(args);
	locTime = localtime(&time);
	strftime(buf,sizeof(buf),"%S",locTime);
	sec = atoi(buf);
        eSymSetIntValue(script,&script->returnValue, sec);
	return(0);
}


int doStrftime(script, args)
	ember	*script;
	e_stack	*args;
{
	time_t	time;
	char	*fmt;
	static	char buf[4096];

        fmt = emTextArg(args);
        time = emIntArg(args->next);

	strftime(buf, 4096, fmt, localtime(&time));
        eSymSetTextValue(script,&script->returnValue, buf);
	return(0);
}
